@@ -9,6 +9,7 @@ import os
 import yaml
 
 from ..mem_definition_base import EfuseBlocksBase, EfuseFieldsBase, EfuseRegistersBase
+from typing import List
 
 
 class EfuseDefineRegisters(EfuseRegistersBase):
@@ -106,7 +107,7 @@ class EfuseDefineBlocks(EfuseBlocksBase):
 
 
 class EfuseDefineFields(EfuseFieldsBase):
-    def __init__(self) -> None:
+    def __init__(self, extend_efuse_table) -> None:
         # List of efuse fields from TRM the chapter eFuse Controller.
         self.EFUSES = []
 
@@ -115,7 +116,7 @@ class EfuseDefineFields(EfuseFieldsBase):
         # if BLK_VERSION_MAJOR is 1, these efuse fields are in BLOCK2
         self.BLOCK2_CALIBRATION_EFUSES = []
 
-        self.CALC = []
+        self.CALC: List = []
 
         dir_name = os.path.dirname(os.path.abspath(__file__))
         dir_name, file_name = os.path.split(dir_name)
@@ -125,7 +126,7 @@ class EfuseDefineFields(EfuseFieldsBase):
         efuse_file = efuse_file.replace("esp32h2beta1", "esp32h2")
         with open(f"{efuse_file}", "r") as r_file:
             e_desc = yaml.safe_load(r_file)
-        super().__init__(e_desc)
+        super().__init__(e_desc, extend_efuse_table)
 
         for i, efuse in enumerate(self.ALL_EFUSES):
             if efuse.name in [
