@@ -10,6 +10,10 @@ import struct
 import sys
 import time
 import zlib
+import itertools
+
+from intelhex import IntelHex
+from serial import SerialException
 
 from .bin_image import ELFFile, ImageSegment, LoadFirmwareImage
 from .bin_image import (
@@ -25,6 +29,7 @@ from .loader import (
     timeout_per_mb,
 )
 from .targets import CHIP_DEFS, CHIP_LIST, ROM_LIST
+from .uf2_writer import UF2Writer
 from .util import (
     FatalError,
     NotImplementedInROMError,
@@ -96,7 +101,7 @@ def detect_chip(
         print("Detecting chip type...", end="")
         chip_id = detect_port.get_chip_id()
         for cls in [
-            n for n in ROM_LIST if n.CHIP_NAME not in ("ESP8266", "ESP32", "ESP32S2")
+            n for n in ROM_LIST if n.CHIP_NAME not in ("ESP8266", "ESP32", "ESP32-S2")
         ]:
             # cmd not supported on ESP8266 and ESP32 + ESP32-S2 doesn't return chip_id
             if chip_id == cls.IMAGE_CHIP_ID:
@@ -112,7 +117,7 @@ def detect_chip(
         else:
             err_msg = f"Unexpected chip ID value {chip_id}."
     except (UnsupportedCommandError, struct.error, FatalError) as e:
-        # UnsupportedCommmanddError: ESP8266/ESP32 ROM
+        # UnsupportedCommandError: ESP8266/ESP32 ROM
         # struct.error: ESP32-S2
         # FatalError: ESP8266/ESP32 STUB
         print(" Unsupported detection protocol, switching and trying again...")
@@ -226,7 +231,7 @@ def detect_flash_size(esp, args=None):
         if flash_size is None:
             flash_size = "4MB"
             print(
-                "Warning: Could not auto-detect Flash size "
+                "WARNING: Could not auto-detect Flash size "
                 f"(FlashID={flash_id:#x}, SizeID={size_id:#x}), defaulting to 4MB"
             )
         else:
@@ -273,49 +278,60 @@ def _update_image_flash_params(esp, address, args, image):
 
     # After the 8-byte header comes the extended header for chips others than ESP8266.
     # The 15th byte of the extended header indicates if the image is protected by
-    # a SHA256 checksum. In that case we should not modify the header because
-    # the checksum check would fail.
-    sha_implies_keep = args.chip != "esp8266" and image[8 + 15] == 1
-
-    def print_keep_warning(arg_to_keep, arg_used):
-        print(
-            "Warning: Image file at {addr} is protected with a hash checksum, "
-            "so not changing the flash {arg} setting. "
-            "Use the --flash_{arg}=keep option instead of --flash_{arg}={arg_orig} "
-            "in order to remove this warning, or use the --dont-append-digest option "
-            "for the elf2image command in order to generate an image file "
-            "without a hash checksum".format(
-                addr=hex(address), arg=arg_to_keep, arg_orig=arg_used
-            )
-        )
+    # a SHA256 checksum. In that case we recalculate the SHA digest after modifying the header.
+    sha_appended = args.chip != "esp8266" and image[8 + 15] == 1
 
     if args.flash_mode != "keep":
-        new_flash_mode = FLASH_MODES[args.flash_mode]
-        if flash_mode != new_flash_mode and sha_implies_keep:
-            print_keep_warning("mode", args.flash_mode)
-        else:
-            flash_mode = new_flash_mode
+        flash_mode = FLASH_MODES[args.flash_mode]
 
     flash_freq = flash_size_freq & 0x0F
     if args.flash_freq != "keep":
-        new_flash_freq = esp.parse_flash_freq_arg(args.flash_freq)
-        if flash_freq != new_flash_freq and sha_implies_keep:
-            print_keep_warning("frequency", args.flash_freq)
-        else:
-            flash_freq = new_flash_freq
+        flash_freq = esp.parse_flash_freq_arg(args.flash_freq)
 
     flash_size = flash_size_freq & 0xF0
     if args.flash_size != "keep":
-        new_flash_size = esp.parse_flash_size_arg(args.flash_size)
-        if flash_size != new_flash_size and sha_implies_keep:
-            print_keep_warning("size", args.flash_size)
-        else:
-            flash_size = new_flash_size
+        flash_size = esp.parse_flash_size_arg(args.flash_size)
 
     flash_params = struct.pack(b"BB", flash_mode, flash_size + flash_freq)
     if flash_params != image[2:4]:
         print("Flash params set to 0x%04x" % struct.unpack(">H", flash_params))
         image = image[0:2] + flash_params + image[4:]
+
+    # recalculate the SHA digest if it was appended
+    if sha_appended:
+        # Since the changes are only made for images located in the bootloader offset,
+        # we can assume that the image is always a bootloader image.
+        # For merged binaries, we check the bootloader SHA when parameters are changed.
+        image_object = esp.BOOTLOADER_IMAGE(io.BytesIO(image))
+        # get the image header, extended header (if present) and data
+        image_data_before_sha = image[: image_object.data_length]
+        # get the image data after the SHA digest (primary for merged binaries)
+        image_data_after_sha = image[
+            (image_object.data_length + image_object.SHA256_DIGEST_LEN) :
+        ]
+
+        sha_digest_calculated = hashlib.sha256(image_data_before_sha).digest()
+        image = bytes(
+            itertools.chain(
+                image_data_before_sha, sha_digest_calculated, image_data_after_sha
+            )
+        )
+
+        # get the SHA digest newly stored in the image and compare it to the calculated one
+        image_stored_sha = image[
+            image_object.data_length : image_object.data_length
+            + image_object.SHA256_DIGEST_LEN
+        ]
+
+        if hexify(sha_digest_calculated) == hexify(image_stored_sha):
+            print("SHA digest in image updated")
+        else:
+            print(
+                "WARNING: SHA recalculation for binary failed!\n"
+                f"\tExpected calculated SHA: {hexify(sha_digest_calculated)}\n"
+                f"\tSHA stored in binary:    {hexify(image_stored_sha)}"
+            )
+
     return image
 
 
@@ -462,19 +478,32 @@ def write_flash(esp, args):
                     "Use --force to override the warning."
                 )
 
-    # verify file sizes fit in flash
-    flash_end = flash_size_bytes(
-        detect_flash_size(esp) if args.flash_size == "keep" else args.flash_size
+    set_flash_size = (
+        flash_size_bytes(args.flash_size)
+        if args.flash_size not in ["detect", "keep"]
+        else None
     )
-    if flash_end is not None:  # Secure download mode
+    if esp.secure_download_mode:
+        flash_end = set_flash_size
+    else:  # Check against real flash chip size if not in SDM
+        flash_end_str = detect_flash_size(esp)
+        flash_end = flash_size_bytes(flash_end_str)
+        if set_flash_size and set_flash_size > flash_end:
+            print(
+                f"WARNING: Set --flash_size {args.flash_size} "
+                f"is larger than the available flash size of {flash_end_str}."
+            )
+
+    # Verify file sizes fit in the set --flash_size, or real flash size if smaller
+    flash_end = min(set_flash_size, flash_end) if set_flash_size else flash_end
+    if flash_end is not None:
         for address, argfile in args.addr_filename:
             argfile.seek(0, os.SEEK_END)
             if address + argfile.tell() > flash_end:
                 raise FatalError(
-                    "File %s (length %d) at offset %d "
-                    "will not fit in %d bytes of flash. "
-                    "Use --flash_size argument, or change flashing address."
-                    % (argfile.name, argfile.tell(), address, flash_end)
+                    f"File {argfile.name} (length {argfile.tell()}) at offset "
+                    f"{address} will not fit in {flash_end} bytes of flash. "
+                    "Change the --flash_size argument, or flashing address."
                 )
             argfile.seek(0)
 
@@ -544,70 +573,118 @@ def write_flash(esp, args):
             print("Will flash %s uncompressed" % argfile.name)
             compress = False
 
-        if args.no_stub:
-            print("Erasing flash...")
-        image = pad_to(
-            argfile.read(), esp.FLASH_ENCRYPTED_WRITE_ALIGN if encrypted else 4
-        )
+        image = argfile.read()
+
         if len(image) == 0:
             print("WARNING: File %s is empty" % argfile.name)
             continue
-        image = _update_image_flash_params(esp, address, args, image)
+
+        image = pad_to(image, esp.FLASH_ENCRYPTED_WRITE_ALIGN if encrypted else 4)
+
+        if args.no_stub:
+            print("Erasing flash...")
+
+            # It is not possible to write to not aligned addresses without stub,
+            # so there are added 0xFF (erase) bytes at the beginning of the image
+            # to align it.
+            bytes_over = address % esp.FLASH_SECTOR_SIZE
+            address -= bytes_over
+            image = b"\xFF" * bytes_over + image
+
+        if not esp.secure_download_mode and not esp.get_secure_boot_enabled():
+            image = _update_image_flash_params(esp, address, args, image)
+        else:
+            print(
+                "WARNING: Security features enabled, so not changing any flash settings."
+            )
         calcmd5 = hashlib.md5(image).hexdigest()
         uncsize = len(image)
         if compress:
             uncimage = image
             image = zlib.compress(uncimage, 9)
-            # Decompress the compressed binary a block at a time,
-            # to dynamically calculate the timeout based on the real write size
-            decompress = zlib.decompressobj()
-            blocks = esp.flash_defl_begin(uncsize, len(image), address)
-        else:
-            blocks = esp.flash_begin(uncsize, address, begin_rom_encrypted=encrypted)
-        argfile.seek(0)  # in case we need it again
-        seq = 0
-        bytes_sent = 0  # bytes sent on wire
-        bytes_written = 0  # bytes written to flash
-        t = time.time()
-
-        timeout = DEFAULT_TIMEOUT
-
-        while len(image) > 0:
-            print_overwrite(
-                "Writing at 0x%08x... (%d %%)"
-                % (address + bytes_written, 100 * (seq + 1) // blocks)
-            )
-            sys.stdout.flush()
-            block = image[0 : esp.FLASH_WRITE_SIZE]
-            if compress:
-                # feeding each compressed block into the decompressor lets us
-                # see block-by-block how much will be written
-                block_uncompressed = len(decompress.decompress(block))
-                bytes_written += block_uncompressed
-                block_timeout = max(
-                    DEFAULT_TIMEOUT,
-                    timeout_per_mb(ERASE_WRITE_TIMEOUT_PER_MB, block_uncompressed),
-                )
-                if not esp.IS_STUB:
-                    timeout = (
-                        block_timeout  # ROM code writes block to flash before ACKing
-                    )
-                esp.flash_defl_block(block, seq, timeout=timeout)
-                if esp.IS_STUB:
-                    # Stub ACKs when block is received,
-                    # then writes to flash while receiving the block after it
-                    timeout = block_timeout
-            else:
-                # Pad the last block
-                block = block + b"\xff" * (esp.FLASH_WRITE_SIZE - len(block))
-                if encrypted:
-                    esp.flash_encrypt_block(block, seq)
+        original_image = image  # Save the whole image in case retry is needed
+        # Try again if reconnect was successful
+        for attempt in range(1, esp.WRITE_FLASH_ATTEMPTS + 1):
+            try:
+                if compress:
+                    # Decompress the compressed binary a block at a time,
+                    # to dynamically calculate the timeout based on the real write size
+                    decompress = zlib.decompressobj()
+                    blocks = esp.flash_defl_begin(uncsize, len(image), address)
                 else:
-                    esp.flash_block(block, seq)
-                bytes_written += len(block)
-            bytes_sent += len(block)
-            image = image[esp.FLASH_WRITE_SIZE :]
-            seq += 1
+                    blocks = esp.flash_begin(
+                        uncsize, address, begin_rom_encrypted=encrypted
+                    )
+                argfile.seek(0)  # in case we need it again
+                seq = 0
+                bytes_sent = 0  # bytes sent on wire
+                bytes_written = 0  # bytes written to flash
+                t = time.time()
+
+                timeout = DEFAULT_TIMEOUT
+
+                while len(image) > 0:
+                    print_overwrite(
+                        "Writing at 0x%08x... (%d %%)"
+                        % (address + bytes_written, 100 * (seq + 1) // blocks)
+                    )
+                    sys.stdout.flush()
+                    block = image[0 : esp.FLASH_WRITE_SIZE]
+                    if compress:
+                        # feeding each compressed block into the decompressor lets us
+                        # see block-by-block how much will be written
+                        block_uncompressed = len(decompress.decompress(block))
+                        bytes_written += block_uncompressed
+                        block_timeout = max(
+                            DEFAULT_TIMEOUT,
+                            timeout_per_mb(
+                                ERASE_WRITE_TIMEOUT_PER_MB, block_uncompressed
+                            ),
+                        )
+                        if not esp.IS_STUB:
+                            timeout = block_timeout  # ROM code writes block to flash before ACKing
+                        esp.flash_defl_block(block, seq, timeout=timeout)
+                        if esp.IS_STUB:
+                            # Stub ACKs when block is received,
+                            # then writes to flash while receiving the block after it
+                            timeout = block_timeout
+                    else:
+                        # Pad the last block
+                        block = block + b"\xff" * (esp.FLASH_WRITE_SIZE - len(block))
+                        if encrypted:
+                            esp.flash_encrypt_block(block, seq)
+                        else:
+                            esp.flash_block(block, seq)
+                        bytes_written += len(block)
+                    bytes_sent += len(block)
+                    image = image[esp.FLASH_WRITE_SIZE :]
+                    seq += 1
+                break
+            except SerialException:
+                if attempt == esp.WRITE_FLASH_ATTEMPTS or encrypted:
+                    # Already retried once or encrypted mode is disabled because of security reasons
+                    raise
+                print("\nLost connection, retrying...")
+                esp._port.close()
+                print("Waiting for the chip to reconnect", end="")
+                for _ in range(DEFAULT_CONNECT_ATTEMPTS):
+                    try:
+                        time.sleep(1)
+                        esp._port.open()
+                        print()  # Print new line which was suppressed by print(".")
+                        esp.connect()
+                        if esp.IS_STUB:
+                            # Hack to bypass the stub overwrite check
+                            esp.IS_STUB = False
+                            # Reflash stub because chip was reset
+                            esp = esp.run_stub()
+                        image = original_image
+                        break
+                    except SerialException:
+                        print(".", end="")
+                        sys.stdout.flush()
+                else:
+                    raise  # Reconnect limit reached
 
         if esp.IS_STUB:
             # Stub only writes each block to flash after 'ack'ing the receive,
@@ -642,7 +719,7 @@ def write_flash(esp, args):
                     print("Flash md5: %s" % res)
                     print(
                         "MD5 of 0xFF is %s"
-                        % (hashlib.md5(b"\xFF" * uncsize).hexdigest())
+                        % (hashlib.md5(b"\xff" * uncsize).hexdigest())
                     )
                     raise FatalError("MD5 of file does not match data in flash!")
                 else:
@@ -790,7 +867,7 @@ def image_info(args):
         format_str = "{:7}  {:#07x}  {:#010x}  {:#010x}  {}"
         app_desc = None
         bootloader_desc = None
-        for idx, seg in enumerate(image.segments, start=1):
+        for idx, seg in enumerate(image.segments):
             segs = seg.get_memory_type(image)
             seg_name = ", ".join(segs)
             if "DROM" in segs:  # The DROM segment starts with the esp_app_desc_t struct
@@ -812,9 +889,11 @@ def image_info(args):
         print(
             "Checksum: {:#02x} ({})".format(
                 image.checksum,
-                "valid"
-                if image.checksum == calc_checksum
-                else "invalid - calculated {:02x}".format(calc_checksum),
+                (
+                    "valid"
+                    if image.checksum == calc_checksum
+                    else "invalid - calculated {:02x}".format(calc_checksum)
+                ),
             )
         )
         try:
@@ -889,8 +968,7 @@ def image_info(args):
             ESP8266V2FirmwareImage.IMAGE_V2_MAGIC,
         ]:
             raise FatalError(
-                "This is not a valid image "
-                "(invalid magic number: {:#x})".format(magic)
+                f"This is not a valid image (invalid magic number: {magic:#x})"
             )
 
         if args.chip == "auto":
@@ -937,9 +1015,11 @@ def image_info(args):
     print(
         "Checksum: {:02x} ({})".format(
             image.checksum,
-            "valid"
-            if image.checksum == calc_checksum
-            else "invalid - calculated {:02x}".format(calc_checksum),
+            (
+                "valid"
+                if image.checksum == calc_checksum
+                else "invalid - calculated {:02x}".format(calc_checksum)
+            ),
         )
     )
     try:
@@ -989,7 +1069,11 @@ def elf2image(args):
         image.min_rev = args.min_rev
         image.min_rev_full = args.min_rev_full
         image.max_rev_full = args.max_rev_full
-        image.append_digest = args.append_digest
+        image.ram_only_header = args.ram_only_header
+        if image.ram_only_header:
+            image.append_digest = False
+        else:
+            image.append_digest = args.append_digest
     elif args.version == "1":  # ESP8266
         image = ESP8266ROMFirmwareImage()
     elif args.version == "2":
@@ -1013,6 +1097,13 @@ def elf2image(args):
         image.elf_sha256 = e.sha256()
         image.elf_sha256_offset = args.elf_sha256_offset
 
+    if args.ram_only_header:
+        print(
+            "Image has only RAM segments visible. "
+            "ROM segments are hidden and SHA256 digest is not appended."
+        )
+        image.sort_segments()
+
     before = len(image.segments)
     image.merge_adjacent_segments()
     if len(image.segments) != before:
@@ -1084,7 +1175,7 @@ def run(esp, args):
     esp.run()
 
 
-def flash_id(esp, args):
+def detect_flash_id(esp):
     flash_id = esp.flash_id()
     print("Manufacturer: %02x" % (flash_id & 0xFF))
     flid_lowbyte = (flash_id >> 16) & 0xFF
@@ -1092,11 +1183,27 @@ def flash_id(esp, args):
     print(
         "Detected flash size: %s" % (DETECTED_FLASH_SIZES.get(flid_lowbyte, "Unknown"))
     )
+
+
+def flash_id(esp, args):
+    detect_flash_id(esp)
     flash_type = esp.flash_type()
     flash_type_dict = {0: "quad (4 data lines)", 1: "octal (8 data lines)"}
     flash_type_str = flash_type_dict.get(flash_type)
     if flash_type_str:
         print(f"Flash type set in eFuse: {flash_type_str}")
+    esp.get_flash_voltage()
+
+
+def read_flash_sfdp(esp, args):
+    detect_flash_id(esp)
+
+    sfdp = esp.read_spiflash_sfdp(args.addr, args.bytes * 8)
+    print(f"SFDP[{args.addr}..{args.addr+args.bytes-1}]: ", end="")
+    for i in range(args.bytes):
+        print(f"{sfdp&0xff:02X} ", end="")
+        sfdp = sfdp >> 8
+    print()
 
 
 def read_flash(esp, args):
@@ -1180,15 +1287,104 @@ def write_flash_status(esp, args):
     print(("After flash status:   " + fmt) % esp.read_status(args.bytes))
 
 
+# The following mapping was taken from the ROM code
+# This mapping is same across all targets in the ROM
+SECURITY_INFO_FLAG_MAP = {
+    "SECURE_BOOT_EN": (1 << 0),
+    "SECURE_BOOT_AGGRESSIVE_REVOKE": (1 << 1),
+    "SECURE_DOWNLOAD_ENABLE": (1 << 2),
+    "SECURE_BOOT_KEY_REVOKE0": (1 << 3),
+    "SECURE_BOOT_KEY_REVOKE1": (1 << 4),
+    "SECURE_BOOT_KEY_REVOKE2": (1 << 5),
+    "SOFT_DIS_JTAG": (1 << 6),
+    "HARD_DIS_JTAG": (1 << 7),
+    "DIS_USB": (1 << 8),
+    "DIS_DOWNLOAD_DCACHE": (1 << 9),
+    "DIS_DOWNLOAD_ICACHE": (1 << 10),
+}
+
+
+# Get the status of respective security flag
+def get_security_flag_status(flag_name, flags_value):
+    try:
+        return (flags_value & SECURITY_INFO_FLAG_MAP[flag_name]) != 0
+    except KeyError:
+        raise ValueError(f"Invalid flag name: {flag_name}")
+
+
 def get_security_info(esp, args):
     si = esp.get_security_info()
-    # TODO: better display
+    print()
+    title = "Security Information:"
+    print(title)
+    print("=" * len(title))
     print("Flags: {:#010x} ({})".format(si["flags"], bin(si["flags"])))
-    print("Flash_Crypt_Cnt: {:#x}".format(si["flash_crypt_cnt"]))
-    print("Key_Purposes: {}".format(si["key_purposes"]))
+    if esp.KEY_PURPOSES:
+        print(f"Key Purposes: {si['key_purposes']}")
+        desc = "\n  ".join(
+            [
+                f"BLOCK_KEY{key_num} - {esp.KEY_PURPOSES.get(purpose, 'UNKNOWN')}"
+                for key_num, purpose in enumerate(si["key_purposes"])
+                if key_num <= esp.EFUSE_MAX_KEY
+            ]
+        )
+        print(f"  {desc}")
     if si["chip_id"] is not None and si["api_version"] is not None:
-        print("Chip_ID: {}".format(si["chip_id"]))
-        print("Api_Version: {}".format(si["api_version"]))
+        print("Chip ID: {}".format(si["chip_id"]))
+        print("API Version: {}".format(si["api_version"]))
+
+    flags = si["flags"]
+
+    if get_security_flag_status("SECURE_BOOT_EN", flags):
+        print("Secure Boot: Enabled")
+        if get_security_flag_status("SECURE_BOOT_AGGRESSIVE_REVOKE", flags):
+            print("Secure Boot Aggressive key revocation: Enabled")
+
+        revoked_keys = []
+        for i, key in enumerate(
+            [
+                "SECURE_BOOT_KEY_REVOKE0",
+                "SECURE_BOOT_KEY_REVOKE1",
+                "SECURE_BOOT_KEY_REVOKE2",
+            ]
+        ):
+            if get_security_flag_status(key, flags):
+                revoked_keys.append(i)
+
+        if len(revoked_keys) > 0:
+            print("Secure Boot Key Revocation Status:\n")
+            for i in revoked_keys:
+                print(f"\tSecure Boot Key{i} is Revoked\n")
+
+    else:
+        print("Secure Boot: Disabled")
+
+    flash_crypt_cnt = bin(si["flash_crypt_cnt"])
+    if (flash_crypt_cnt.count("1") % 2) != 0:
+        print("Flash Encryption: Enabled")
+    else:
+        print("Flash Encryption: Disabled")
+
+    CRYPT_CNT_STRING = "SPI Boot Crypt Count (SPI_BOOT_CRYPT_CNT)"
+    if esp.CHIP_NAME == "esp32":
+        CRYPT_CNT_STRING = "Flash Crypt Count (FLASH_CRYPT_CNT)"
+
+    print(f"{CRYPT_CNT_STRING}: {si['flash_crypt_cnt']:#x}")
+
+    if get_security_flag_status("DIS_DOWNLOAD_DCACHE", flags):
+        print("Dcache in UART download mode: Disabled")
+
+    if get_security_flag_status("DIS_DOWNLOAD_ICACHE", flags):
+        print("Icache in UART download mode: Disabled")
+
+    hard_dis_jtag = get_security_flag_status("HARD_DIS_JTAG", flags)
+    soft_dis_jtag = get_security_flag_status("SOFT_DIS_JTAG", flags)
+    if hard_dis_jtag:
+        print("JTAG: Permanently Disabled")
+    elif soft_dis_jtag:
+        print("JTAG: Software Access Disabled")
+    if get_security_flag_status("DIS_USB", flags):
+        print("USB Access: Disabled")
 
 
 def merge_bin(args):
@@ -1198,9 +1394,9 @@ def merge_bin(args):
         msg = (
             "Please specify the chip argument"
             if args.chip == "auto"
-            else "Invalid chip choice: '{}'".format(args.chip)
+            else f"Invalid chip choice: '{args.chip}'"
         )
-        msg = msg + " (choose from {})".format(", ".join(CHIP_LIST))
+        msg = f"{msg} (choose from {', '.join(CHIP_LIST)})"
         raise FatalError(msg)
 
     # sort the files by offset.
@@ -1211,31 +1407,64 @@ def merge_bin(args):
     first_addr = input_files[0][0]
     if first_addr < args.target_offset:
         raise FatalError(
-            "Output file target offset is 0x%x. Input file offset 0x%x is before this."
-            % (args.target_offset, first_addr)
+            f"Output file target offset is {args.target_offset:#x}. "
+            f"Input file offset {first_addr:#x} is before this."
         )
 
-    if args.format != "raw":
-        raise FatalError(
-            "This version of esptool only supports the 'raw' output format"
+    if args.format == "uf2":
+        with UF2Writer(
+            chip_class.UF2_FAMILY_ID,
+            args.output,
+            args.chunk_size,
+            md5_enabled=not args.md5_disable,
+        ) as writer:
+            for addr, argfile in input_files:
+                print(f"Adding {argfile.name} at {addr:#x}")
+                image = argfile.read()
+                image = _update_image_flash_params(chip_class, addr, args, image)
+                writer.add_file(addr, image)
+        print(
+            f"Wrote {os.path.getsize(args.output):#x} bytes to file {args.output}, "
+            f"ready to be flashed with any ESP USB Bridge"
         )
 
-    with open(args.output, "wb") as of:
+    elif args.format == "raw":
+        with open(args.output, "wb") as of:
 
-        def pad_to(flash_offs):
-            # account for output file offset if there is any
-            of.write(b"\xFF" * (flash_offs - args.target_offset - of.tell()))
+            def pad_to(flash_offs):
+                # account for output file offset if there is any
+                of.write(b"\xff" * (flash_offs - args.target_offset - of.tell()))
 
+            for addr, argfile in input_files:
+                pad_to(addr)
+                image = argfile.read()
+                image = _update_image_flash_params(chip_class, addr, args, image)
+                of.write(image)
+            if args.fill_flash_size:
+                pad_to(flash_size_bytes(args.fill_flash_size))
+            print(
+                f"Wrote {of.tell():#x} bytes to file {args.output}, "
+                f"ready to flash to offset {args.target_offset:#x}"
+            )
+    elif args.format == "hex":
+        out = IntelHex()
+        if len(input_files) == 1:
+            print(
+                "WARNING: Only one input file specified, output may include "
+                "additional padding if input file was previously merged. "
+                "Please refer to the documentation for more information: "
+                "https://docs.espressif.com/projects/esptool/en/latest/esptool/basic-commands.html#hex-output-format"  # noqa E501
+            )
         for addr, argfile in input_files:
-            pad_to(addr)
+            ihex = IntelHex()
             image = argfile.read()
             image = _update_image_flash_params(chip_class, addr, args, image)
-            of.write(image)
-        if args.fill_flash_size:
-            pad_to(flash_size_bytes(args.fill_flash_size))
+            ihex.frombytes(image, addr)
+            out.merge(ihex)
+        out.write_hex_file(args.output)
         print(
-            "Wrote 0x%x bytes to file %s, ready to flash to offset 0x%x"
-            % (of.tell(), args.output, args.target_offset)
+            f"Wrote {os.path.getsize(args.output):#x} bytes to file {args.output}, "
+            f"ready to flash to offset {args.target_offset:#x}"
         )
 
 
